
function NFMieProgram

% NFMIEPROGRAM Help data to come

% Author: David Rioux
% Version number: 012Beta
% Date modified: 2017-XX-XX
% Note: Corrected the calculation of the mean of the nth power of the
%       surface field.
%       Moved the GUI construction into a separate file.
% To add: Add the possibility to save the spectra of the mean of the nth
%         power of the surface field.
%         Add the NPPhotothermia module
% 
% Copyright 2012-2017 David Rioux.

% Create the main handles structure
h=[];
% Create the GUI
[ h ] = generate_Mainfigure( h );


% Finish GUI initialization
BtlImg=load('NPBottle.mat');
S=BtlImg.S;
V=BtlImg.V;

SpectralOrTemporalStatus=0;
MeanEFieldStatus=0;
global MeanEn;
global NPabs;
global NPData;

% Trace the initial graphical elements
UpdateGraphics;

% Make the GUI visible
set(h.Mainfigure,'Visible','on');



function [ h ] = generate_Mainfigure( h )
% Initialize and hide the GUI as it is being constructed.
h.Mainfigure = figure('Name','NFMieProgram',...
    'NumberTitle','off',...
    'Position',[300 100 1000 600],...
    'Color',[0.9 0.9 0.9],...
    'Visible','off',...
    'MenuBar','none');

%  Construct the components.
h.NP_panel = uipanel('Parent',h.Mainfigure,...
    'Title','Nanostructure',...
    'Units','pixels',...
    'Position',[5 435 455 165],...
    'BackgroundColor',[0.9 0.9 0.9]);
h.NPType_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Type',...
    'Units','pixels',...
    'Position',[5 130 85 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','left');
h.NPType_popup = uicontrol(h.NP_panel,...
    'Style','popupmenu',...
    'String',{'Nanoparticle','Core-Shell'},...
    'Units','pixels',...
    'Position',[5 110 85 20],...
    'Callback',{@NPType_popup_Callback});
h.NPCoreMaterial_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Material',...
    'Units','pixels',...
    'Position',[95 130 120 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','left');
h.NPCoreMaterial_popup = uicontrol(h.NP_panel,...
    'Style','popupmenu',...
    'String',{'Au','AuAg Alloy','Ag','Al','SiO2','TiO2(o)','TiO2(e)','TiO2(avg)','File','Custom'},...
    'Units','pixels',...
    'Position',[95 110 120 20],...
    'Callback',{@NPCoreMaterial_popup_Callback});
h.NPCore_n_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Real (n) ',...
    'Units','pixels',...
    'Position',[95 80 70 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right',...
    'Visible','off');
h.NPCore_n_edit = uicontrol(h.NP_panel,...
    'Style','edit',...
    'String','1.00',...
    'Units','pixels',...
    'Position',[165 80 50 25],...
    'Visible','off',...
    'Callback',{@UpdateGraphics});
h.NPCore_k_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Imaginary (k) ',...
    'Units','pixels',...
    'Position',[95 55 70 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right',...
    'Visible','off');
h.NPCore_k_edit = uicontrol(h.NP_panel,...
    'Style','edit',...
    'String','0.00',...
    'Units','pixels',...
    'Position',[165 55 50 25],...
    'Visible','off',...
    'Callback',{@UpdateGraphics});
h.NPShellMaterial_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Shell material',...
    'Units','pixels',...
    'Position',[220 130 120 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','left',...
    'Visible','off');
h.NPShellMaterial_popup = uicontrol(h.NP_panel,...
    'Style','popupmenu',...
    'String',{'Au','AuAg Alloy','Ag','Al','SiO2','TiO2(o)','TiO2(e)','TiO2(avg)','File','Custom'},...
    'Units','pixels',...
    'Position',[220 110 120 20],...
    'Visible','off',...
    'Callback',{@NPShellMaterial_popup_Callback});
h.NPShell_n_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Real (n) ',...
    'Units','pixels',...
    'Position',[220 80 70 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right',...
    'Visible','off');
h.NPShell_n_edit = uicontrol(h.NP_panel,...
    'Style','edit',...
    'String','1.00',...
    'Units','pixels',...
    'Position',[290 80 50 25],...
    'Visible','off',...
    'Callback',{@UpdateGraphics});
h.NPShell_k_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Imaginary (k) ',...
    'Units','pixels',...
    'Position',[220 55 70 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right',...
    'Visible','off');
h.NPShell_k_edit = uicontrol(h.NP_panel,...
    'Style','edit',...
    'String','0.00',...
    'Units','pixels',...
    'Position',[290 55 50 25],...
    'Visible','off',...
    'Callback',{@UpdateGraphics});
h.Medium_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Medium',...
    'Units','pixels',...
    'Position',[345 130 100 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','left');
h.Medium_popup = uicontrol(h.NP_panel,...
    'Style','popupmenu',...
    'String',{'Water','Air','Custom'},...
    'Units','pixels',...
    'Position',[345 110 100 20],...
    'Callback',{@Medium_popup_Callback});
h.Medium_n_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Index (n) ',...
    'Units','pixels',...
    'Position',[345 80 50 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right',...
    'Visible','off');
h.Medium_n_edit = uicontrol(h.NP_panel,...
    'Style','edit',...
    'String','1.00',...
    'Units','pixels',...
    'Position',[395 80 50 25],...
    'Visible','off',...
    'Callback',{@UpdateGraphics});
h.NPCoreDiam_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Diameter (nm) ',...
    'Units','pixels',...
    'Position',[95 30 120 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','center');
h.NPCoreDiam_edit = uicontrol(h.NP_panel,...
    'Style','edit',...
    'String','30',...
    'Units','pixels',...
    'Position',[95 5 120 25],...
    'Callback',{@NPCoreDiam_edit_Callback});
h.NPOuterDiam_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Outer Diameter (nm) ',...
    'Units','pixels',...
    'Position',[220 30 120 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','center',...
    'Visible','off');
h.NPOuterDiam_edit = uicontrol(h.NP_panel,...
    'Style','edit',...
    'String','50',...
    'Units','pixels',...
    'Position',[220 5 120 25],...
    'Visible','off',...
    'Callback',{@NPOuterDiam_edit_Callback});
h.NPShellThick_text = uicontrol(h.NP_panel,...
    'Style','text',...
    'String','Shell Thickness (nm) ',...
    'Units','pixels',...
    'Position',[340 30 110 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','center',...
    'Visible','off');
h.NPShellThick_edit = uicontrol(h.NP_panel,...
    'Style','edit',...
    'String','10',...
    'Units','pixels',...
    'Position',[345 5 100 25],...
    'Visible','off',...
    'Callback',{@NPShellThick_edit_Callback});
h.SizeCorr_check = uicontrol(h.NP_panel,...
    'Style','checkbox',...
    'String','<html>Use size<br/>correction</html>',...
    'Value',1,...
    'Position',[5 5 85 30],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'Visible','off',...
    'Callback',{@UpdateGraphics});

h.SpectraCalc_panel = uibuttongroup('Parent',h.Mainfigure,...
    'Title','Spectra Calculations',...
    'Units','pixels',...
    'Position',[465 470 220 130],...
    'BackgroundColor',[0.9 0.9 0.9]);
h.Sigma_Radio = uicontrol(h.SpectraCalc_panel,...
    'Style','radiobutton',...
    'String','<html>&sigma</html>',...
    'Value',1,...
    'Units','pixels',...
    'Position',[150 85 70 25],...
    'BackgroundColor',[0.9 0.9 0.9]);
h.Q_Radio = uicontrol(h.SpectraCalc_panel,...
    'Style','radiobutton',...
    'String','Q',...
    'Value',0,...
    'Units','pixels',...
    'Position',[150 60 70 25],...
    'BackgroundColor',[0.9 0.9 0.9]);
h.SigmaVol_Radio = uicontrol(h.SpectraCalc_panel,...
    'Style','radiobutton',...
    'String','<html>&sigma / NP<sub>Vol</sub></html>',...
    'Value',0,...
    'Units','pixels',...
    'Position',[150 35 70 25],...
    'BackgroundColor',[0.9 0.9 0.9]);
h.LambdaMin_text = uicontrol(h.SpectraCalc_panel,...
    'Style','text',...
    'String','Lambda min (nm) ',...
    'Units','pixels',...
    'Position',[5 85 90 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.LambdaMin_edit = uicontrol(h.SpectraCalc_panel,...
    'Style','edit',...
    'String','300',...
    'Units','pixels',...
    'Position',[95 85 50 25],...
    'Callback',{@UpdateGraphics});
h.DeltaLambda_text = uicontrol(h.SpectraCalc_panel,...
    'Style','text',...
    'String','Delta lambda (nm) ',...
    'Units','pixels',...
    'Position',[5 60 90 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.DeltaLambda_edit = uicontrol(h.SpectraCalc_panel,...
    'Style','edit',...
    'String','1',...
    'Units','pixels',...
    'Position',[95 60 50 25],...
    'Callback',{@UpdateGraphics});
h.LambdaMax_text = uicontrol(h.SpectraCalc_panel,...
    'Style','text',...
    'String','Lambda max (nm) ',...
    'Units','pixels',...
    'Position',[5 35 90 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.LambdaMax_edit = uicontrol(h.SpectraCalc_panel,...
    'Style','edit',...
    'String','800',...
    'Units','pixels',...
    'Position',[95 35 50 25],...
    'Callback',{@UpdateGraphics});
h.EfieldLambda_text = uicontrol(h.SpectraCalc_panel,...
    'Style','text',...
    'String','Lambda (nm) ',...
    'Units','pixels',...
    'Position',[5 5 90 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.EfieldLambda_edit = uicontrol(h.SpectraCalc_panel,...
    'Style','edit',...
    'String','800',...
    'Units','pixels',...
    'Position',[95 5 50 25]);
h.EfieldCalc_button = uicontrol(h.SpectraCalc_panel,...
    'Style','pushbutton',...
    'String','Compute E-field',...
    'Units','pixels',...
    'Position',[150 5 60 30],...
    'Callback',{@EfieldCalc_button_Callback});

h.TempCalc_panel = uibuttongroup('Parent',h.Mainfigure,...
    'Title','Temperature Calculation',...
    'Units','pixels',...
    'Position',[465 420 180 180],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'Visible','off');
h.LaserIntensity_text = uicontrol(h.TempCalc_panel,...
    'Style','text',...
    'String','Laser intensity (W/cm) ',...
    'Units','pixels',...
    'Position',[5 135 120 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.LaserIntensity_edit = uicontrol(h.TempCalc_panel,...
    'Style','edit',...
    'String','6e5',...
    'Units','pixels',...
    'Position',[125 135 50 25]);
h.PulseTime_text = uicontrol(h.TempCalc_panel,...
    'Style','text',...
    'String','Pulse time (s) ',...
    'Units','pixels',...
    'Position',[5 110 120 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.PulseTime_edit = uicontrol(h.TempCalc_panel,...
    'Style','edit',...
    'String','30e-6',...
    'Units','pixels',...
    'Position',[125 110 50 25]);
h.TotalTime_text = uicontrol(h.TempCalc_panel,...
    'Style','text',...
    'String','Total time (s) ',...
    'Units','pixels',...
    'Position',[5 85 120 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.TotalTime_edit = uicontrol(h.TempCalc_panel,...
    'Style','edit',...
    'String','60e-6',...
    'Units','pixels',...
    'Position',[125 85 50 25],...
    'Callback',{@TotalTime_edit_Callback});
h.DeltaTime_text = uicontrol(h.TempCalc_panel,...
    'Style','text',...
    'String','Time step (s) ',...
    'Units','pixels',...
    'Position',[5 60 120 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.DeltaTime_edit = uicontrol(h.TempCalc_panel,...
    'Style','edit',...
    'String','1e-7',...
    'Units','pixels',...
    'Position',[125 60 50 25]);
h.LambdaLaser_text = uicontrol(h.TempCalc_panel,...
    'Style','text',...
    'String','Laser wavelength (nm) ',...
    'Units','pixels',...
    'Position',[5 35 120 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.LambdaLaser_edit = uicontrol(h.TempCalc_panel,...
    'Style','edit',...
    'String','800',...
    'Units','pixels',...
    'Position',[125 35 50 25],...
    'Callback',{@UpdateGraphics});
h.TempCalculation_button = uicontrol(h.TempCalc_panel,...
    'Style','pushbutton',...
    'String','Compute temperature',...
    'Units','pixels',...
    'Position',[25 5 130 25],...
    'Callback',{@TempCalculation_button_Callback});
h.NPAbs_axes = axes('Parent',h.Mainfigure,...
    'Units','pixels',...
    'Position',[725 460 250 120],...
    'Visible','off');
axis([0, 800, 0 100]);
xlabel('Wavelength (nm)','BackgroundColor',[0.9 0.9 0.9]);
ylabel('\sigma_a_b_s (nm)','BackgroundColor',[0.9 0.9 0.9]);

h.SurfaceFieldCalc_panel = uipanel('Parent',h.Mainfigure,...
    'Title','Mean Surface Field Calculations',...
    'Units','pixels',...
    'Position',[465 400 290 70],...
    'BackgroundColor',[0.9 0.9 0.9]);
h.FieldExponent_text = uicontrol(h.SurfaceFieldCalc_panel,...
    'Style','text',...
    'String','n = ',...
    'Units','pixels',...
    'Position',[5 25 90 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');
h.FieldExponent_edit = uicontrol(h.SurfaceFieldCalc_panel,...
    'Style','edit',...
    'String','2',...
    'Units','pixels',...
    'Position',[95 25 50 25]);
h.FieldExponent_slider = uicontrol(h.SurfaceFieldCalc_panel,...
    'Style', 'slider',...
    'Min',1.0,...
    'Max',15.0,...
    'Value',2,...
    'SliderStep',[1/14 1/14],...
    'Units','pixels',...
    'Position',[5 5 140 20],...
    'Callback',{@FieldExponent_slider_Callback});
h.MeanField_button = uicontrol(h.SurfaceFieldCalc_panel,...
    'Style','pushbutton',...
    'String','<html>Compute surface |E-field|<sup>n</sup></html>',...
    'Units','pixels',...
    'Position',[150 5 135 50],...
    'Callback',{@MeanField_button_Callback});


h.ExtMax_text = uicontrol('Style','text',...
    'String','Extinction max (nm) ',...
    'Units','pixels',...
    'Position',[760 435 250 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','left');
h.ScaMax_text = uicontrol('Style','text',...
    'String','Scattering max (nm) ',...
    'Units','pixels',...
    'Position',[760 415 250 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','left');
h.AbsMax_text = uicontrol('Style','text',...
    'String','Absorption max (nm) ',...
    'Units','pixels',...
    'Position',[760 395 250 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','left');



h.SpectraAndTemporal_axes = axes('Units','pixels',...
    'Position',[65 105 415 275]);
h.SpectraAndTemporal_Outerpos=get(h.SpectraAndTemporal_axes,'OuterPosition');
axis([300, 800, 0 1000]);
xlabel('Wavelength (nm)','BackgroundColor',[0.9 0.9 0.9]);
ylabel('Cross section (nm)','BackgroundColor',[0.9 0.9 0.9]);

h.eFieldAndSpatial_axes = axes('Units','pixels',...
    'Position',[565 105 415 275]);
h.eFieldAndSpatial_Outerpos=get(h.eFieldAndSpatial_axes,'OuterPosition');
axis([0, 10000, 0 100]);
xlabel('Radial Position (nm)','BackgroundColor',[0.9 0.9 0.9]);
ylabel('\Delta T (K)','BackgroundColor',[0.9 0.9 0.9]);

h.ImageTemp_axes = axes('Units','pixels',...
    'Position',[780 180 200 200],...
    'Visible','off');


h.PositionVisualization_text = uicontrol(h.Mainfigure,...
    'Style','text',...
    'String','Position (nm) ',...
    'Units','pixels',...
    'Position',[5 35 90 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right',...
    'Visible','off');
h.PositionVisualization_edit = uicontrol(h.Mainfigure,...
    'Style','edit',...
    'String','0',...
    'Units','pixels',...
    'Position',[95 35 50 25],...
    'Visible','off',...
    'Callback',{@Visualization_edit_Callback});
h.PositionVisualization_slider = uicontrol(h.Mainfigure,...
    'Style', 'slider',...
    'Min',0,...
    'Max',60e-6,...
    'Value',0,...
    'SliderStep',[0.001 0.01],...
    'Units','pixels',...
    'Position',[160 35 300 20],...
    'Visible','off',...
    'Callback',{@Visualization_slider_Callback});




h.TimeVisualization_text = uicontrol(h.Mainfigure,...
    'Style','text',...
    'String','Time (s) ',...
    'Units','pixels',...
    'Position',[500 35 90 20],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right',...
    'Visible','off');
h.TimeVisualization_edit = uicontrol(h.Mainfigure,...
    'Style','edit',...
    'String','0',...
    'Units','pixels',...
    'Position',[590 35 50 25],...
    'Visible','off',...
    'Callback',{@Visualization_edit_Callback});
h.TimeVisualization_slider = uicontrol(h.Mainfigure,...
    'Style', 'slider',...
    'Min',0,...
    'Max',60e-6,...
    'Value',0,...
    'SliderStep',[0.001 0.01],...
    'Units','pixels',...
    'Position',[660 35 300 20],...
    'Visible','off',...
    'Callback',{@Visualization_slider_Callback});






h.Save_button = uicontrol('Style','pushbutton',...
    'String','Save',...
    'Units','pixels',...
    'Position',[20 10 100 25],...
    'Callback',{@Save_button_Callback});
h.Save_edit = uicontrol('Style','edit',...
    'String','Enter desired filename',...
    'Units','pixels',...
    'Position',[120,10,200,25]);


h.TransColor_text = uicontrol('Style','text',...
    'String','Transmitted color',...
    'Units','pixels',...
    'Position',[750 580 100 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','center');
h.TransColor_axes = axes('Units','pixels',...
    'Position',[750 480 100 100]);
h.ScattColor_text = uicontrol('Style','text',...
    'String','Scattered color',...
    'Units','pixels',...
    'Position',[875 580 100 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','center');
h.ScattColor_axes = axes('Units','pixels',...
    'Position',[875 480 100 100]);

h.SwitchPanel_button = uicontrol('Style','pushbutton',...
    'String','Switch to temperature calculation',...
    'Units','pixels',...
    'Position',[450 5 200 25],...
    'Callback',{@SwitchPanel_Callback});

h.VersionNumber_text = uicontrol('Style','text',...
    'String','Version 012B',...
    'Units','pixels',...
    'Position',[925 5 70 15],...
    'BackgroundColor',[0.9 0.9 0.9],...
    'HorizontalAlignment','right');

end



% --- Executes on selection change in h.NPType_popup.
function NPType_popup_Callback(hObject, ~)
    index_selected=get(hObject,'Value');
    if index_selected==2
        set(h.NPCoreMaterial_text,'String','Core material');
        set(h.NPCoreDiam_text,'String','Core diameter (nm)');
        set(h.NPShellMaterial_text,'Visible','on');
        set(h.NPShellMaterial_popup,'Visible','on');
        if get(h.NPShellMaterial_popup,'Value')==2
            set(h.NPShell_n_text,'Visible','on');
            set(h.NPShell_n_edit,'Visible','on');
            set(h.SizeCorr_check,'Visible','on');
        elseif get(h.NPShellMaterial_popup,'Value')==10
            set(h.NPShell_n_text,'Visible','on');
            set(h.NPShell_n_edit,'Visible','on');
            set(h.NPShell_k_text,'Visible','on');
            set(h.NPShell_k_edit,'Visible','on');
        end
        set(h.NPOuterDiam_text,'Visible','on');
        set(h.NPOuterDiam_edit,'Visible','on');
        set(h.NPShellThick_text,'Visible','on');
        set(h.NPShellThick_edit,'Visible','on');
    else
        set(h.NPCoreMaterial_text,'String','Material');
        set(h.NPCoreDiam_text,'String','Diameter (nm)');
        set(h.NPShellMaterial_text,'Visible','off');
        set(h.NPShellMaterial_popup,'Visible','off');
        set(h.NPShell_n_text,'Visible','off');
        set(h.NPShell_n_edit,'Visible','off');
        set(h.NPShell_k_text,'Visible','off');
        set(h.NPShell_k_edit,'Visible','off');
        set(h.NPOuterDiam_text,'Visible','off');
        set(h.NPOuterDiam_edit,'Visible','off');
        set(h.NPShellThick_text,'Visible','off');
        set(h.NPShellThick_edit,'Visible','off');
    end
    coreindex_selected=get(h.NPCoreMaterial_popup,'Value');
    if index_selected==1 && coreindex_selected~=2
        set(h.SizeCorr_check,'Visible','off');
    end
    
    UpdateGraphics;
end



% --- Executes on selection change in NPCoreMaterial_popup.
function NPCoreMaterial_popup_Callback(hObject, ~)
    global core_filename;
    index_selected=get(hObject,'Value');
    if index_selected==2    % Alloy
        set(h.NPCore_n_text,'String','GMF','Visible','on');
        set(h.NPCore_n_edit,'String','0.5','Visible','on');
        set(h.NPCore_k_text,'Visible','off');
        set(h.NPCore_k_edit,'Visible','off');
        set(h.SizeCorr_check,'Visible','on');
    elseif index_selected==9    % External file
        set(h.NPShell_n_text,'Visible','off');
        set(h.NPShell_n_edit,'Visible','off');
        set(h.NPShell_k_text,'Visible','off');
        set(h.NPShell_k_edit,'Visible','off');
        [core_filename,~,~] = uigetfile('IndexFiles/*.*',...
            'Please choose appropriate file');
    elseif index_selected==10    % Custom
        set(h.NPCore_n_text,'String','Real (n) ','Visible','on');
        set(h.NPCore_n_edit,'String','1.00','Visible','on');
        set(h.NPCore_k_text,'Visible','on');
        set(h.NPCore_k_edit,'Visible','on');
    else
        set(h.NPCore_n_text,'Visible','off');
        set(h.NPCore_n_edit,'Visible','off');
        set(h.NPCore_k_text,'Visible','off');
        set(h.NPCore_k_edit,'Visible','off');
    end
    NPTypeindex_selected=get(h.NPType_popup,'Value');
    shellindex_selected=get(h.NPShellMaterial_popup,'Value');
    if index_selected~=2 && (NPTypeindex_selected==1 || shellindex_selected~=2)
        set(h.SizeCorr_check,'Visible','off');
    end
    
    UpdateGraphics;
end



% --- Executes on selection change in NPShellMaterial_popup.
function NPShellMaterial_popup_Callback(hObject, ~)
    global shell_filename;
    index_selected=get(hObject,'Value');
    if index_selected==2    % Alloy
        set(h.NPShell_n_text,'String','GMF','Visible','on');
        set(h.NPShell_n_edit,'String','0.5','Visible','on');
        set(h.NPShell_k_text,'Visible','off');
        set(h.NPShell_k_edit,'Visible','off');
        set(h.SizeCorr_check,'Visible','on');
    elseif index_selected==9    % External file
        set(h.NPShell_n_text,'Visible','off');
        set(h.NPShell_n_edit,'Visible','off');
        set(h.NPShell_k_text,'Visible','off');
        set(h.NPShell_k_edit,'Visible','off');
        [shell_filename,~,~] = uigetfile('IndexFiles/*.*',...
            'Please choose appropriate file');
    elseif index_selected==10    % Custom
        set(h.NPShell_n_text,'String','Real (n) ','Visible','on');
        set(h.NPShell_n_edit,'String','1.00','Visible','on');
        set(h.NPShell_k_text,'Visible','on');
        set(h.NPShell_k_edit,'Visible','on');
    else
        set(h.NPShell_n_text,'Visible','off');
        set(h.NPShell_n_edit,'Visible','off');
        set(h.NPShell_k_text,'Visible','off');
        set(h.NPShell_k_edit,'Visible','off');
    end
    coreindex_selected=get(h.NPCoreMaterial_popup,'Value');
    if index_selected~=2 && coreindex_selected~=2
        set(h.SizeCorr_check,'Visible','off');
    end
    
    UpdateGraphics;
end



% --- Executes on selection change in Medium_popup.
function Medium_popup_Callback(hObject, ~)
    index_selected=get(hObject,'Value');
    if index_selected==3
        set(h.Medium_n_text,'Visible','on');
        set(h.Medium_n_edit,'Visible','on');
    else
        set(h.Medium_n_text,'Visible','off');
        set(h.Medium_n_edit,'Visible','off');
    end
    
    UpdateGraphics;
end



% --- Executes on change in NPCoreDiam_edit or NPOuterDiam_edit.
function NPCoreDiam_edit_Callback(hObject, ~)
    CoreDiam=str2double(hObject.String);
    OuterDiam=str2double(h.NPOuterDiam_edit.String);
    if CoreDiam>OuterDiam
        OuterDiam=CoreDiam;
        h.NPOuterDiam_edit.String=num2str(OuterDiam);
    end
    h.NPShellThick_edit.String=num2str((OuterDiam-CoreDiam)/2);
    
    UpdateGraphics;
end



% --- Executes on change in NPOuterDiam_edit.
function NPOuterDiam_edit_Callback(hObject, ~)
    CoreDiam=str2double(h.NPCoreDiam_edit.String);
    OuterDiam=str2double(hObject.String);
    if CoreDiam>OuterDiam
        CoreDiam=OuterDiam;
        h.NPCoreDiam_edit.String=num2str(CoreDiam);
    end
    set(h.NPShellThick_edit,'String',num2str((OuterDiam-CoreDiam)/2));
    
    UpdateGraphics;
end



% --- Executes on change in NPShellThick_edit.
function NPShellThick_edit_Callback(hObject, ~)
    CoreDiam=str2double(h.NPCoreDiam_edit.String);
    ShellThick=str2double(hObject.String);
    h.NPOuterDiam_edit.String=num2str(CoreDiam+2*ShellThick);
    
    UpdateGraphics;
end



% --- Executes on button press in Calculation_button.
function Calculation_button_Callback(~, ~)
    NP_Type=get(h.NPType_popup,'Value');
    if NP_Type==1
        diam=str2double(get(h.NPCoreDiam_edit,'String'));
        LambdaMin=str2double(get(h.LambdaMin_edit,'String'));
        DeltaLambda=str2double(get(h.DeltaLambda_edit,'String'));
        LambdaMax=str2double(get(h.LambdaMax_edit,'String'));
        lambda=(LambdaMin:DeltaLambda:LambdaMax)';
        core_index_selected=get(h.NPCoreMaterial_popup,'Value');
        [core_nindex]=get_nindex(1,core_index_selected,lambda);
        medium_index_selected=get(h.Medium_popup,'Value');
        [medium_nindex]=get_medium_nindex(medium_index_selected,lambda);
        [abs,sca,ext]=NFMieCoated(diam,[core_nindex,medium_nindex],lambda,1);
    else
        CoreDiam=str2double(get(h.NPCoreDiam_edit,'String'));
        ShellThick=str2double(get(h.NPShellThick_edit,'String'));
        diam=[CoreDiam CoreDiam+2*ShellThick];
        LambdaMin=str2double(get(h.LambdaMin_edit,'String'));
        DeltaLambda=str2double(get(h.DeltaLambda_edit,'String'));
        LambdaMax=str2double(get(h.LambdaMax_edit,'String'));
        lambda=(LambdaMin:DeltaLambda:LambdaMax)';
        core_index_selected=get(h.NPCoreMaterial_popup,'Value');
        [core_nindex]=get_nindex(1,core_index_selected,lambda);
        shell_index_selected=get(h.NPShellMaterial_popup,'Value');
        [shell_nindex]=get_nindex(2,shell_index_selected,lambda);
        medium_index_selected=get(h.Medium_popup,'Value');
        [medium_nindex]=get_medium_nindex(medium_index_selected,lambda);
        [abs,sca,ext]=NFMieCoated(diam,...
            [core_nindex,shell_nindex,medium_nindex],lambda,1);
    end
    global lambda_global abs_global sca_global ext_global;
    lambda_global=lambda;
    abs_global=abs;
    sca_global=sca;
    ext_global=ext;
    axes(h.SpectraAndTemporal_axes);
    delete(findall(gcf,'Tag','AnnotationForEFieldView'))
    cla reset;
    set(h.SpectraAndTemporal_axes,'Units','pixels',...
        'OuterPosition',h.SpectraAndTemporal_Outerpos);
    hold on;
    if get(h.Sigma_Radio,'Value')==1
        h1=plot(lambda,abs,'linewidth',1.5,'color',[1 0 0]);    %,'LineSmoothing','on');
        h2=plot(lambda,sca,'linewidth',1.5,'color',[0 1 0]);    %,'LineSmoothing','on');
        h3=plot(lambda,ext,'linewidth',1.5,'color',[0 0 1]);    %,'LineSmoothing','on');
        
        h4=plot(str2double(h.EfieldLambda_edit.String)*[1 1], [0 1.05*max(ext)],'r:');
        
        axis([min(lambda), max(lambda), 0 1.05*max(ext)]);
        ylabel('Cross section (nm)');
    elseif get(h.Q_Radio,'Value')==1
        h1=plot(lambda,abs/((pi/4)*diam(end)^2),'linewidth',1.5,'color',[1 0 0]);    %,'LineSmoothing','on');
        h2=plot(lambda,sca/((pi/4)*diam(end)^2),'linewidth',1.5,'color',[0 1 0]);    %,'LineSmoothing','on');
        h3=plot(lambda,ext/((pi/4)*diam(end)^2),'linewidth',1.5,'color',[0 0 1]);    %,'LineSmoothing','on');
        axis([min(lambda), max(lambda), 0 1.05*max(ext/((pi/4)*diam(end)^2))]);
        ylabel('Efficiency');
    else
        h1=plot(lambda,abs/((pi/6)*diam(end)^3),'linewidth',1.5,'color',[1 0 0]);    %,'LineSmoothing','on');
        h2=plot(lambda,sca/((pi/6)*diam(end)^3),'linewidth',1.5,'color',[0 1 0]);    %,'LineSmoothing','on');
        h3=plot(lambda,ext/((pi/6)*diam(end)^3),'linewidth',1.5,'color',[0 0 1]);    %,'LineSmoothing','on');
        axis([min(lambda), max(lambda), 0 1.05*max(ext/((pi/6)*diam(end)^3))]);
        ylabel('Volumetric cross section (1/nm)');
    end
    hold off;
    set(gca,'FontSize',10);
    legend([h3 h2 h1],{'Extinction';'Scattering';'Absorption'},...
        'location','best');
    legend boxoff;
    grid on;
    xlabel('Wavelength (nm)');
    [maxext,maxposext]=max(ext);
    set(h.ExtMax_text,'String',['Max extinction @'...
        num2str(lambda(maxposext)) 'nm = ' num2str(maxext,'%.3g') 'nm']);
    [maxsca,maxpossca]=max(sca);
    set(h.ScaMax_text,'String',['Max scattering @'...
        num2str(lambda(maxpossca)) 'nm = ' num2str(maxsca,'%.3g') 'nm']);
    [maxabs,maxposabs]=max(abs);
    set(h.AbsMax_text,'String',['Max absorption @'...
        num2str(lambda(maxposabs)) 'nm = ' num2str(maxabs,'%.3g') 'nm']);
%     set(h.AbsMax_edit,'String',num2str(lambda(maxposabs)));
    
    % Calculation of the transmitted and scattered colors
%     ExtSpectrum=-log(0.1)*2.5*ext./max(ext);
    ExtSpectrum=-log(0.1)*2.5*20*((1/diam(end)).^3)*ext;
    [ TransImg, ScatImg ] = ScatExtSpectra2sRGB(lambda, ExtSpectrum, sca);
    image(TransImg, 'Parent', h.TransColor_axes);
    set(h.TransColor_axes,'Visible','off');
    image(ScatImg, 'Parent', h.ScattColor_axes);
    set(h.ScattColor_axes,'Visible','off');
end



% --- Executes on button press in EfieldCalc_button.
function EfieldCalc_button_Callback(~, ~)
    NP_Type=get(h.NPType_popup,'Value');
    if NP_Type==1
        diam=str2double(get(h.NPCoreDiam_edit,'String'));
        lambda=str2double(get(h.EfieldLambda_edit,'String'));
        core_index_selected=get(h.NPCoreMaterial_popup,'Value');
        [core_nindex]=get_nindex(1,core_index_selected,lambda);
        medium_index_selected=get(h.Medium_popup,'Value');
        [medium_nindex]=get_medium_nindex(medium_index_selected,lambda);
        rads=[linspace(0.001*diam,0.499*diam,20), 0.5*diam, linspace(0.501*diam,2*diam,20)];
        dtheta=pi/60;
        theta=0:dtheta:pi;
        [ETot]=NFMieCoated(diam,[core_nindex,medium_nindex],lambda,3, 'dphi', pi/2, 'shellradius', rads, 'dtheta', dtheta);
    else
        CoreDiam=str2double(get(h.NPCoreDiam_edit,'String'));
        ShellThick=str2double(get(h.NPShellThick_edit,'String'));
        diam=[CoreDiam CoreDiam+2*ShellThick];
        lambda=str2double(get(h.EfieldLambda_edit,'String'));
        core_index_selected=get(h.NPCoreMaterial_popup,'Value');
        [core_nindex]=get_nindex(1,core_index_selected,lambda);
        shell_index_selected=get(h.NPShellMaterial_popup,'Value');
        [shell_nindex]=get_nindex(2,shell_index_selected,lambda);
        medium_index_selected=get(h.Medium_popup,'Value');
        [medium_nindex]=get_medium_nindex(medium_index_selected,lambda);
        rads=[linspace(0.001*diam(1),0.499*diam(1),20), 0.5*diam(1), linspace(0.501*diam(1),0.499*diam(2),20), 0.5*diam(2), linspace(0.501*diam(2),2*diam(2),20)];
        dtheta=pi/60;
        theta=0:dtheta:pi;
        [ETot]=NFMieCoated(diam,...
            [core_nindex,shell_nindex,medium_nindex],lambda,3, 'dphi', pi/2, 'shellradius', rads, 'dtheta', dtheta);
    end
    
    ETot=squeeze(ETot(:,1,:));
    ETot=[ETot(end:-1:1,2) ETot];
    rads=[-rads(1) rads];
    [R, T]=meshgrid(rads, theta);
    [Zq,Xq] = meshgrid(-diam(end):diam(end)/100:diam(end),0:diam(end)/100:diam(end));
    Rq=sqrt(Zq.^2 + Xq.^2);
    Tq=atan2(Xq,Zq);
    ETotq = interp2(R,T,ETot,Rq,Tq,'linear');
%     ETotq=[ETotq(end:-1:2,:); ETotq];

    
    
% rads=[1:1:28 linspace(28.1,31.9,10) 32:1:100];
% dtheta=pi/60;
% theta=0:dtheta:pi;
% [ ETot3 ] = v025_NFMieCoated( [30 60], [ 1.45 0.15 + 1i*3.5, 1.33 ], 800, 3, 'dphi', pi/2, 'shellradius', rads, 'dtheta', dtheta);
% ETot3=squeeze(ETot3(:,1,:));
% ETot3=[ETot3(end:-1:1,2) ETot3];
% rads=[-rads(1) rads];
% [R, T]=meshgrid(rads, theta);
% [Zq,Xq] = meshgrid(-60:60/200:60,0:60/200:60);
% Rq=sqrt(Zq.^2 + Xq.^2);
% Tq=atan2(Xq,Zq);
% 
% ETot3q = interp2(R,T,ETot3,Rq,Tq,'linear');
% 
% ETot3q=[ETot3q(end:-1:2,:); ETot3q];
% 
% figure;
% pcolor(ETot3q); shading flat; colormap(parula(1024));
% axis equal;
    
    
    
%     Centerpos=ceil(size(ETot,2)/2);
%     ETot(1,Centerpos)=0.25*(2*ETot(2,Centerpos)+ETot(1,Centerpos-1)+ETot(1,Centerpos+1));
    axes(h.eFieldAndSpatial_axes);
    cla reset;
    set(h.eFieldAndSpatial_axes,'Units','pixels',...
        'OuterPosition',h.eFieldAndSpatial_Outerpos);
    hold on;
    h1=pcolor(-diam(end):diam(end)/100:diam(end),...
        -diam(end):diam(end)/100:diam(end),[ETotq(end:-1:2,:);ETotq]);
    colormap(parula(2048)); shading('flat');
    annotation('arrow',[0.63 0.63],[0.55 0.62],'headstyle','vback2',...
        'headlength',10,'headwidth',10,'linewidth',1,'color',[0 0 0],...
        'Tag','AnnotationForEFieldView');
    annotation('textbox',[0.60 0.56 0.05 0.05],'string','E','fontsize',12,...
        'fontweight','bold','color',[0 0 0],'linestyle','none',...
        'Tag','AnnotationForEFieldView');
    annotation('arrow',[0.63 0.68],[0.55 0.55],'headstyle','vback2',...
        'headlength',10,'headwidth',10,'linewidth',1,'color',[1 0 0],...
        'Tag','AnnotationForEFieldView');
    annotation('textbox',[0.64 0.50 0.05 0.05],'string','k','fontsize',12,...
        'fontweight','bold','color',[1 0 0],'linestyle','none',...
        'Tag','AnnotationForEFieldView');
    axis image;
    colorbar;
    hold off;

    
    % Code for tracing sphere
%     ETot2=[ETot ETot(:,end-1:-1:1) ETot(:,2:1:end) ETot(:,end-1:-1:1)];
%     [Phi,Theta] = meshgrid(0:pi/60:2*pi, 0:pi/60:pi);
%     X=sin(Theta).*cos(Phi);
%     Y=sin(Theta).*sin(Phi);
%     Z=cos(Theta);
%     surf(X,Y,Z,ETot2); shading interp; axis equal;
%     set(gcf,'Toolbar','figure');

%     set(gca,'FontSize',10);
%     xlabel('Wavelength (nm)');
%     ylabel('Cross section (nm)');
    
end



% --- Executes on change in FieldExponent_slider.
function FieldExponent_slider_Callback(hObject, ~)
    set(h.FieldExponent_edit,'String',get(hObject,'value'));

    if MeanEFieldStatus==1
        LambdaMin=str2double(get(h.LambdaMin_edit,'String'));
        DeltaLambda=str2double(get(h.DeltaLambda_edit,'String'));
        LambdaMax=str2double(get(h.LambdaMax_edit,'String'));
        lambda=(LambdaMin:DeltaLambda:LambdaMax)';
        n=get(h.FieldExponent_slider,'value');
        axes(h.SpectraAndTemporal_axes);
        hold off;
        h1=plot(lambda,MeanEn(:,n),'linewidth',1.5,'color',[0 0 0]);
        set(gca,'FontSize',10);
        grid on;
        xlabel('Wavelength (nm)');
        if n<10
            ylabel(['|E-field^' num2str(n) '|']);
        else
            nstring=num2str(n);
            ylabel(['|E-field^' nstring(1) '^' nstring(2) '|']);
        end
    end
end



% --- Executes on button press in MeanField_button.
function MeanField_button_Callback(~, ~)
    dtheta=pi/120;
    theta=(0:dtheta:pi)';
    dphi=pi/120;
    phi=0:dphi:pi/2;
    theta=repmat(theta,1,length(phi));
    LambdaMin=str2double(get(h.LambdaMin_edit,'String'));
    DeltaLambda=str2double(get(h.DeltaLambda_edit,'String'));
    LambdaMax=str2double(get(h.LambdaMax_edit,'String'));
    lambda=(LambdaMin:DeltaLambda:LambdaMax)';
    MeanEn=zeros(length(lambda),15);
    NP_Type=get(h.NPType_popup,'Value');
    WtBr = waitbar(0,'Please wait...');
    if NP_Type==1
        diam=str2double(get(h.NPCoreDiam_edit,'String'));
        core_index_selected=get(h.NPCoreMaterial_popup,'Value');
        [core_nindex]=get_nindex(1,core_index_selected,lambda);
        medium_index_selected=get(h.Medium_popup,'Value');
        [medium_nindex]=get_medium_nindex(medium_index_selected,lambda);
        for m=1:length(lambda)
            [ ETot ] = NFMieCoated( diam, [ core_nindex(m), medium_nindex(m) ], lambda(m), 3, 'dtheta', dtheta, 'dphi', dphi );
            for n=1:15
                MeanEn(m,n)=mean(mean((ETot.^n).*sin(theta)));
            end
            waitbar(m/length(lambda));
        end
    else
        CoreDiam=str2double(get(h.NPCoreDiam_edit,'String'));
        ShellThick=str2double(get(h.NPShellThick_edit,'String'));
        diam=[CoreDiam CoreDiam+2*ShellThick];
        core_index_selected=get(h.NPCoreMaterial_popup,'Value');
        [core_nindex]=get_nindex(1,core_index_selected,lambda);
        shell_index_selected=get(h.NPShellMaterial_popup,'Value');
        [shell_nindex]=get_nindex(2,shell_index_selected,lambda);
        medium_index_selected=get(h.Medium_popup,'Value');
        [medium_nindex]=get_medium_nindex(medium_index_selected,lambda);
        for m=1:length(lambda)
            [ ETot ] = NFMieCoated( diam, [ core_nindex(m), shell_nindex(m), medium_nindex(m) ], lambda(m), 3, 'dtheta', dtheta, 'dphi', dphi );
            for n=1:15
                MeanEn(m,n)=mean(mean((ETot.^n).*sin(theta)));
            end
            waitbar(m/length(lambda));
        end
    end
    close(WtBr);
    
    MeanEFieldStatus=1;
    n=get(h.FieldExponent_slider,'value');
    axes(h.SpectraAndTemporal_axes);
    hold off;
    h1=plot(lambda,MeanEn(:,n),'linewidth',1.5,'color',[0 0 0]);
    set(gca,'FontSize',10);
    grid on;
    xlabel('Wavelength (nm)');
    if n<10
        ylabel(['|E-field^' num2str(n) '|']);
    else
        nstring=num2str(n);
        ylabel(['|E-field^' nstring(1) '^' nstring(2) '|']);
    end
end



% --- Executes on button press in Save_button.
function Save_button_Callback(~, ~)
    global lambda_global abs_global sca_global ext_global;
    filename=get(h.Save_edit,'String');
    fid=fopen(filename,'w');
    
    fprintf(fid,'Spectra savefile\r\n');
    fprintf(fid,'=======================================================\r\n');
    fprintf(fid,'The format is :\r\n');
    fprintf(fid,'Column 1 ==> Wavelength (nm)\r\n');
    fprintf(fid,'Column 2 ==> Absorption (nm)\r\n');
    fprintf(fid,'Column 3 ==> Scattering (nm)\r\n');
    fprintf(fid,'Column 4 ==> Extinction (nm)\r\n');
    fprintf(fid,'=======================================================\r\n');
    for k=1:length(lambda_global)
        fprintf(fid,'%5.3f %10.10e %10.10e %10.10e\r\n',[lambda_global(k) ...
            abs_global(k) sca_global(k) ext_global(k)]);
    end
    fclose(fid);
end



% --- Executes on press of Spectral vs Temporal button.
function SwitchPanel_Callback(~, ~)
    if SpectralOrTemporalStatus == 0
        SpectralOrTemporalStatus=1;
        % --- Hiding elements of the GUI
        h.SpectraCalc_panel.Visible = 'off';
        h.SurfaceFieldCalc_panel.Visible = 'off';
        h.TransColor_text.Visible = 'off';
        h.TransColor_axes.Visible = 'off';
        cla(h.TransColor_axes);
        h.ScattColor_text.Visible = 'off';
        h.ScattColor_axes.Visible = 'off';
        cla(h.ScattColor_axes);
        h.ExtMax_text.Visible = 'off';
        h.ScaMax_text.Visible = 'off';
%         h.AbsMax_text.Visible = 'off';
        
        % --- Showing elements of the GUI
        h.TempCalc_panel.Visible = 'on';
        h.NPAbs_axes.Visible = 'on';
        cla(h.SpectraAndTemporal_axes,'reset');
        set(h.SpectraAndTemporal_axes,'Units','pixels',...
            'OuterPosition',h.SpectraAndTemporal_Outerpos);
        xlabel(h.SpectraAndTemporal_axes,'Time (s)');
        ylabel(h.SpectraAndTemporal_axes,'\Delta T (K)');
        
        cla(h.eFieldAndSpatial_axes,'reset');
        delete(findall(gcf,'Tag','AnnotationForEFieldView'));
        xlabel(h.eFieldAndSpatial_axes,'Radial position (nm)');
        ylabel(h.eFieldAndSpatial_axes,'\Delta T (K)');
        
        h.PositionVisualization_text.Visible = 'on';
        h.PositionVisualization_edit.Visible = 'on';
        h.PositionVisualization_slider.Visible = 'on';
        h.TimeVisualization_text.Visible = 'on';
        h.TimeVisualization_edit.Visible = 'on';
        h.TimeVisualization_slider.Visible = 'on';
        
        
        h.SwitchPanel_button.String = 'Switch to spectrum calculation';
        
        UpdateGraphics;
    else
        SpectralOrTemporalStatus=0;
        % --- Hiding elements of the GUI
        h.TempCalc_panel.Visible = 'off';
        cla(h.NPAbs_axes,'reset');
        h.NPAbs_axes.Visible = 'off';
        cla(h.eFieldAndSpatial_axes,'reset');
        h.eFieldAndSpatial_axes.Visible = 'off';
        cla(h.ImageTemp_axes,'reset');
        h.ImageTemp_axes.Visible = 'off';
        
        h.PositionVisualization_text.Visible = 'off';
        h.PositionVisualization_edit.Visible = 'off';
        h.PositionVisualization_slider.Visible = 'off';
        h.TimeVisualization_text.Visible = 'off';
        h.TimeVisualization_edit.Visible = 'off';
        h.TimeVisualization_slider.Visible = 'off';
        
        
        % --- Showing elements of the GUI
        h.SpectraCalc_panel.Visible = 'on';
        h.SurfaceFieldCalc_panel.Visible = 'on';
        h.TransColor_text.Visible = 'on';
        h.ScattColor_text.Visible = 'on';
        h.ExtMax_text.Visible = 'on';
        h.ScaMax_text.Visible = 'on';
        h.AbsMax_text.Visible = 'on';
        
        
        xlabel(h.SpectraAndTemporal_axes,'Wavelength (nm)');
        ylabel(h.SpectraAndTemporal_axes,'Cross section (nm)');
        xlabel(h.eFieldAndSpatial_axes,[]);
        ylabel(h.eFieldAndSpatial_axes,[]);
        
        
        h.SwitchPanel_button.String = 'Switch to temperature calculation';
        UpdateGraphics;
    end
end



% --- Executes on modification of TotalTime_edit
function TotalTime_edit_Callback(~, ~)
    TotalTime=str2double(h.TotalTime_edit.String);
    DeltaTime=str2double(h.DeltaTime_edit.String);
    if TotalTime<100*DeltaTime
        h.DeltaTime_edit.String=num2str(TotalTime/100);
    end
end



% --- Executes on press of TempCalculation_button.
function TempCalculation_button_Callback(~, ~)
    if h.NPType_popup.Value==1
        NPData.diameters=str2double(h.NPCoreDiam_edit.String);
        NPData.abs=NPabs;
        CoreMaterial=h.NPCoreMaterial_popup.String(h.NPCoreMaterial_popup.Value);
        [NPData.k(1), NPData.c(1), NPData.rho(1)] = get_NPData(CoreMaterial);
        MediumMaterial=h.Medium_popup.String(h.Medium_popup.Value);
        [NPData.k(2), NPData.c(2), NPData.rho(2)] = get_NPData(MediumMaterial);
        NPData.G=1.05e-10;
    else
        NPData.diameters=[str2double(h.NPCoreDiam_edit.String) str2double(h.NPOuterDiam_edit.String)];
        NPData.abs=[0 NPabs];
        CoreMaterial=h.NPCoreMaterial_popup.String(h.NPCoreMaterial_popup.Value);
        [NPData.k(1), NPData.c(1), NPData.rho(1)] = get_NPData(CoreMaterial);
        ShellMaterial=h.NPShellMaterial_popup.String(h.NPShellMaterial_popup.Value);
        [NPData.k(2), NPData.c(2), NPData.rho(2)] = get_NPData(ShellMaterial);
        MediumMaterial=h.Medium_popup.String(h.Medium_popup.Value);
        [NPData.k(3), NPData.c(3), NPData.rho(3)] = get_NPData(MediumMaterial);
        NPData.G=[2e-10 1.05e-10];
    end

    dr = 0.1;
    
    ILaser = 1e-14 * str2double(get(h.LaserIntensity_edit,'String'));
    tPulse = str2double(get(h.PulseTime_edit,'String'));
    tSim = str2double(get(h.TotalTime_edit,'String'));
    dt = str2double(get(h.DeltaTime_edit,'String'));
    
    [NPData.T,NPData.r,NPData.time] = NPPhotothermia(NPData,ILaser,tPulse,tSim,dr,dt);      % Running temperature calculation
    
    if h.NPType_popup.Value==1      % Correcting the radial position vector for subsequent interpolation
        pos=find(NPData.r>=0.5*NPData.diameters,1);
        NPData.r(pos+1)=NPData.r(pos+1)+eps(NPData.r(pos+1));
    else
        pos=find(NPData.r>=0.5*NPData.diameters(1),1);
        NPData.r(pos+1)=NPData.r(pos+1)+eps(NPData.r(pos+1));
        pos=find(NPData.r>=0.5*NPData.diameters(2),1);
        NPData.r(pos+1)=NPData.r(pos+1)+eps(NPData.r(pos+1));
    end

    TraceTempResults(1,1);
    
    set(h.PositionVisualization_slider,'Max',size(NPData.T,1)-1);
    set(h.PositionVisualization_slider,'SliderStep',[1/(size(NPData.T,1)-1) 10/(size(NPData.T,1)-1)]);
    set(h.PositionVisualization_slider,'Value',0);
    h.PositionVisualization_edit.String = 0;
    
    set(h.TimeVisualization_slider,'Max',size(NPData.T,2)-1);
    set(h.TimeVisualization_slider,'SliderStep',[1/(size(NPData.T,2)-1) 10/(size(NPData.T,2)-1)]);
    set(h.TimeVisualization_slider,'Value',0);
    h.TimeVisualization_edit.String = 0;
end



% --- Executes on modification of Visualization_edit
function Visualization_edit_Callback(~, ~)
    PosVisu = str2double(h.PositionVisualization_edit.String);
    [~, SpatialIndex] = min(abs(NPData.r-PosVisu));
    h.PositionVisualization_edit.String = NPData.r(SpatialIndex);
    h.PositionVisualization_slider.Value = SpatialIndex-1;
    
    TimeVisu = str2double(h.TimeVisualization_edit.String);
    [~, TimeIndex] = min(abs(NPData.time-TimeVisu));
    h.TimeVisualization_edit.String = NPData.time(TimeIndex);
    h.TimeVisualization_slider.Value = TimeIndex-1;
    
    TraceTempResults(SpatialIndex,TimeIndex);
end



% --- Executes on modification of Visualization_slider
function Visualization_slider_Callback(~, ~)
    SpatialIndex = round(get(h.PositionVisualization_slider,'value')+1);
    h.PositionVisualization_edit.String = NPData.r(SpatialIndex);
    
    TimeIndex=round(get(h.TimeVisualization_slider,'value')+1);
    h.TimeVisualization_edit.String = NPData.time(TimeIndex);
    
    TraceTempResults(SpatialIndex,TimeIndex);
end














% --- Executes on modifications in NP_Panel
function UpdateGraphics(~,~)
    if SpectralOrTemporalStatus == 0
        Calculation_button_Callback;
        EfieldCalc_button_Callback;
    else
        AbsorptionCalculation;
        TempCalculation_button_Callback;
    end
end



function [nindex] = get_nindex(volume,indice,lambda)
    global core_filename shell_filename;
    if indice==1    % Gold (Data by Johnson & Christy)
        wavelength=[187.9;191.6;195.3;199.3;203.3;207.3;211.9;216.4;...
            221.4;226.2;231.3;237.1;242.6;249.0;255.1;261.6;268.9;276.1;...
            284.4;292.4;300.9;310.7;320.4;331.5;342.5;354.2;367.9;381.5;...
            397.4;413.3;430.5;450.9;471.4;495.9;520.9;548.6;582.1;616.8;...
            659.5;704.5;756.0;821.1;892.0;984.0;1088.0;1216.0;1393.0;...
            1610.0;1937.0];
        ngold_data=[1.28;1.32;1.34;1.33;1.33;1.30;1.30;1.30;1.30;1.31;...
            1.30;1.32;1.32;1.33;1.33;1.35;1.38;1.43;1.47;1.49;1.53;1.53;...
            1.54;1.48;1.48;1.50;1.48;1.46;1.47;1.46;1.45;1.38;1.31;1.04;...
            0.62;0.43;0.29;0.21;0.14;0.13;0.14;0.16;0.17;0.22;0.27;0.35;...
            0.43;0.56;0.92];
        kgold_data=[1.188;1.203;1.226;1.251;1.277;1.304;1.350;1.387;...
            1.427;1.460;1.497;1.536;1.577;1.631;1.688;1.749;1.803;1.847;...
            1.869;1.878;1.889;1.893;1.898;1.883;1.871;1.866;1.895;1.933;...
            1.952;1.958;1.948;1.914;1.849;1.833;2.081;2.455;2.863;3.272;...
            3.697;4.103;4.542;5.083;5.663;6.350;7.150;8.145;9.519;...
            11.210;13.780];
        nindex=spline(wavelength,ngold_data,lambda);
        kindex=spline(wavelength,kgold_data,lambda);
        nindex=nindex+1i*kindex;
    elseif indice==2    % AuAg Alloy (Model by Rioux et al. 2014)
        if volume==1
            if get(h.SizeCorr_check,'Value')
                [epsilon]=AuAgAlloyDielectricModel(...
                    str2double(get(h.NPCore_n_edit,'String')),...
                    'lambda', lambda,...
                    'size', str2double(get(h.NPCoreDiam_edit,'String')));
            else
                [epsilon]=AuAgAlloyDielectricModel(...
                    str2double(get(h.NPCore_n_edit,'String')),...
                    'lambda', lambda);
            end
        elseif volume==2
            if get(h.SizeCorr_check,'Value')
                [epsilon]=AuAgAlloyDielectricModel(...
                    str2double(get(h.NPShell_n_edit,'String')),...
                    'lambda', lambda,...
                    'size', str2double(get(h.NPShellThick_edit,'String')));
            else
                [epsilon]=AuAgAlloyDielectricModel(...
                    str2double(get(h.NPShell_n_edit,'String')),...
                    'lambda', lambda);
            end
        end
        nindex=sqrt(epsilon);
    elseif indice==3    % Silver (Data by Palik between ~190 and ~1900 nm)
        wavelength=[187.9;193.7;196.8;200.0;203.3;206.6;213.8;221.4;...
            229.6;238.4;248.0;253.0;258.3;263.8;269.5;275.5;281.8;288.3;...
            295.2;298.8;302.4;306.1;310.0;311.5;313.9;315.5;317.9;319.5;...
            322.0;323.7;326.3;330.6;332.4;335.1;339.7;344.4;354.2;364.7;...
            375.7;387.5;400.0;413.3;427.5;442.8;459.2;476.9;495.9;516.6;...
            539.1;563.6;590.4;619.9;652.6;688.8;729.3;774.9;826.6;885.6;...
            953.7;1033.0;1127.0;1240.0;1378.0;1459.0;1550.0;1653.0;...
            1771.0;1907.0];
        nsilver_data=[0.995;1.028;1.048;1.072;1.098;1.125;1.173;1.208;...
            1.238;1.265;1.298;1.320;1.343;1.372;1.404;1.441;1.476;1.502;...
            1.519;1.522;1.496;1.432;1.323;1.246;1.149;1.044;0.932;0.815;...
            0.708;0.616;0.526;0.371;0.321;0.294;0.259;0.238;0.209;0.186;...
            0.200;0.192;0.173;0.173;0.160;0.157;0.144;0.132;0.130;0.130;...
            0.129;0.120;0.121;0.131;0.140;0.140;0.148;0.143;0.145;0.163;...
            0.198;0.226;0.251;0.329;0.396;0.446;0.514;0.624;0.844;0.873];
        ksilver_data=[1.130;1.180;1.210;1.240;1.260;1.270;1.290;1.300;...
            1.310;1.330;1.350;1.350;1.350;1.350;1.330;1.310;1.260;1.190;...
            1.080;0.992;0.882;0.766;0.647;0.586;0.540;0.514;0.504;0.526;...
            0.565;0.609;0.663;0.813;0.902;0.986;1.120;1.240;1.440;1.610;...
            1.670;1.810;1.950;2.110;2.260;2.400;2.560;2.720;2.880;3.070;...
            3.250;3.450;3.660;3.880;4.150;4.440;4.740;5.090;5.500;5.950;...
            6.430;6.990;7.670;8.490;9.480;10.100;10.800;11.500;12.200;...
            13.300];
        nindex=spline(wavelength,nsilver_data,lambda);
        kindex=spline(wavelength,ksilver_data,lambda);
        nindex=nindex+1i*kindex;
    elseif indice==4    % Aluminium (Data by McPeak between ~150 and ~1700 nm)
        wavelength=[150.0;155.0;160.0;165.0;170.0;175.0;180.0;185.0;190.0;...
            195.0;200.0;205.0;210.0;215.0;220.0;225.0;230.0;235.0;240.0;...
            245.0;250.0;255.0;260.0;265.0;270.0;275.0;280.0;285.0;290.0;...
            295.0;300.0;305.0;310.0;315.0;320.0;325.0;330.0;335.0;340.0;...
            345.0;350.0;355.0;360.0;365.0;370.0;375.0;380.0;385.0;390.0;...
            395.0;400.0;405.0;410.0;415.0;420.0;425.0;430.0;435.0;440.0;...
            445.0;450.0;455.0;460.0;465.0;470.0;475.0;480.0;485.0;490.0;...
            495.0;500.0;505.0;510.0;515.0;520.0;525.0;530.0;535.0;540.0;...
            545.0;550.0;555.0;560.0;565.0;570.0;575.0;580.0;585.0;590.0;...
            595.0;600.0;605.0;610.0;615.0;620.0;625.0;630.0;635.0;640.0;...
            645.0;650.0;655.0;660.0;665.0;670.0;675.0;680.0;685.0;690.0;...
            695.0;700.0;705.0;710.0;715.0;720.0;725.0;730.0;735.0;740.0;...
            745.0;750.0;755.0;760.0;765.0;770.0;775.0;780.0;785.0;790.0;...
            795.0;800.0;805.0;810.0;815.0;820.0;825.0;830.0;835.0;840.0;...
            845.0;850.0;855.0;860.0;865.0;870.0;875.0;880.0;885.0;890.0;...
            895.0;900.0;905.0;910.0;915.0;920.0;925.0;930.0;935.0;940.0;...
            945.0;950.0;955.0;960.0;965.0;970.0;975.0;980.0;985.0;990.0;...
            995.0;1000.0;1005.0;1010.0;1015.0;1020.0;1025.0;1030.0;1035.0;...
            1040.0;1045.0;1050.0;1055.0;1060.0;1065.0;1070.0;1075.0;...
            1080.0;1085.0;1090.0;1095.0;1100.0;1105.0;1110.0;1115.0;...
            1120.0;1125.0;1130.0;1135.0;1140.0;1145.0;1150.0;1155.0;...
            1160.0;1165.0;1170.0;1175.0;1180.0;1185.0;1190.0;1195.0;...
            1200.0;1205.0;1210.0;1215.0;1220.0;1225.0;1230.0;1235.0;...
            1240.0;1245.0;1250.0;1255.0;1260.0;1265.0;1270.0;1275.0;...
            1280.0;1285.0;1290.0;1295.0;1300.0;1305.0;1310.0;1315.0;...
            1320.0;1325.0;1330.0;1335.0;1340.0;1345.0;1350.0;1355.0;...
            1430.0;1435.0;1440.0;1445.0;1450.0;1455.0;1460.0;1465.0;...
            1470.0;1475.0;1480.0;1485.0;1490.0;1495.0;1500.0;1505.0;...
            1510.0;1515.0;1520.0;1525.0;1530.0;1535.0;1540.0;1545.0;...
            1550.0;1555.0;1560.0;1565.0;1570.0;1575.0;1580.0;1585.0;...
            1590.0;1595.0;1600.0;1605.0;1610.0;1615.0;1620.0;1625.0;...
            1630.0;1635.0;1640.0;1645.0;1650.0;1655.0;1660.0;1665.0;...
            1670.0;1675.0;1680.0;1685.0;1690.0;1695.0;1700.0];
        nalu_data=[0.095;0.096;0.099;0.099;0.101;0.107;0.100;0.108;0.107;...
            0.112;0.111;0.112;0.114;0.116;0.116;0.120;0.124;0.129;0.133;...
            0.140;0.141;0.149;0.151;0.161;0.165;0.172;0.179;0.186;0.189;...
            0.197;0.205;0.210;0.219;0.224;0.238;0.245;0.252;0.259;0.267;...
            0.275;0.283;0.292;0.300;0.309;0.318;0.327;0.336;0.346;0.355;...
            0.365;0.375;0.385;0.396;0.407;0.418;0.430;0.441;0.453;0.464;...
            0.477;0.489;0.501;0.515;0.528;0.540;0.555;0.568;0.583;0.597;...
            0.611;0.626;0.640;0.656;0.673;0.688;0.704;0.721;0.738;0.754;...
            0.772;0.789;0.808;0.829;0.849;0.867;0.888;0.909;0.928;0.949;...
            0.971;0.992;1.016;1.038;1.062;1.088;1.113;1.136;1.166;1.190;...
            1.219;1.246;1.275;1.304;1.334;1.365;1.396;1.426;1.457;1.493;...
            1.527;1.560;1.596;1.632;1.670;1.707;1.745;1.786;1.828;1.872;...
            1.917;1.958;2.005;2.054;2.100;2.145;2.190;2.232;2.275;2.315;...
            2.350;2.374;2.399;2.409;2.414;2.410;2.399;2.375;2.342;2.302;...
            2.258;2.205;2.148;2.093;2.031;1.974;1.915;1.855;1.803;1.750;...
            1.695;1.647;1.602;1.557;1.517;1.484;1.447;1.415;1.385;1.356;...
            1.326;1.302;1.280;1.260;1.238;1.218;1.201;1.182;1.168;1.154;...
            1.141;1.127;1.115;1.103;1.094;1.087;1.081;1.071;1.062;1.055;...
            1.052;1.043;1.036;1.033;1.027;1.026;1.024;1.020;1.015;1.014;...
            1.012;1.010;1.008;1.005;1.004;1.004;1.004;1.004;1.005;1.003;...
            1.004;1.005;1.004;1.009;1.008;1.008;1.013;1.015;1.015;1.018;...
            1.018;1.022;1.023;1.024;1.026;1.030;1.035;1.039;1.040;1.043;...
            1.047;1.050;1.051;1.056;1.057;1.061;1.066;1.071;1.073;1.078;...
            1.081;1.087;1.091;1.094;1.097;1.104;1.108;1.109;1.112;1.124;...
            1.129;1.133;1.136;1.209;1.216;1.222;1.226;1.232;1.235;1.240;...
            1.246;1.256;1.252;1.265;1.265;1.275;1.278;1.286;1.290;1.297;...
            1.304;1.314;1.316;1.325;1.333;1.338;1.342;1.347;1.354;1.362;...
            1.368;1.375;1.375;1.386;1.390;1.400;1.398;1.401;1.403;1.421;...
            1.425;1.436;1.439;1.446;1.448;1.456;1.459;1.470;1.478;1.475;...
            1.478;1.462;1.426;1.427;1.455;1.514;1.556;1.584];
        kalu_data=[1.284;1.337;1.403;1.466;1.533;1.597;1.658;1.734;1.791;...
            1.853;1.909;1.970;2.028;2.092;2.152;2.213;2.275;2.337;2.396;...
            2.457;2.515;2.574;2.633;2.693;2.750;2.811;2.869;2.923;2.982;...
            3.042;3.101;3.157;3.215;3.267;3.324;3.379;3.437;3.494;3.550;...
            3.607;3.664;3.720;3.776;3.832;3.889;3.946;4.003;4.058;4.114;...
            4.170;4.226;4.281;4.337;4.391;4.447;4.503;4.559;4.614;4.669;...
            4.724;4.778;4.833;4.888;4.943;4.997;5.051;5.106;5.159;5.213;...
            5.266;5.320;5.375;5.428;5.482;5.536;5.588;5.642;5.694;5.746;...
            5.799;5.852;5.905;5.958;6.009;6.060;6.113;6.165;6.216;6.267;...
            6.317;6.369;6.417;6.467;6.517;6.567;6.615;6.663;6.711;6.759;...
            6.807;6.852;6.898;6.942;6.988;7.032;7.071;7.116;7.158;7.197;...
            7.236;7.274;7.311;7.346;7.380;7.414;7.447;7.475;7.503;7.528;...
            7.552;7.571;7.585;7.599;7.605;7.612;7.611;7.604;7.589;7.572;...
            7.548;7.523;7.487;7.449;7.411;7.369;7.326;7.288;7.253;7.226;...
            7.204;7.188;7.178;7.181;7.185;7.199;7.218;7.243;7.274;7.310;...
            7.350;7.393;7.439;7.490;7.543;7.593;7.648;7.703;7.756;7.812;...
            7.872;7.929;7.985;8.046;8.105;8.164;8.223;8.279;8.338;8.400;...
            8.458;8.512;8.572;8.630;8.688;8.748;8.810;8.868;8.920;8.978;...
            9.037;9.084;9.149;9.204;9.264;9.319;9.373;9.429;9.480;9.537;...
            9.593;9.650;9.704;9.751;9.806;9.860;9.917;9.968;10.024;10.075;...
            10.127;10.183;10.237;10.286;10.341;10.392;10.442;10.493;...
            10.552;10.605;10.658;10.706;10.758;10.808;10.859;10.911;...
            10.962;11.013;11.068;11.116;11.170;11.216;11.267;11.316;...
            11.361;11.411;11.473;11.519;11.563;11.618;11.668;11.720;...
            11.774;11.820;11.864;11.919;11.967;12.015;12.064;12.116;...
            12.165;12.218;12.267;12.992;13.040;13.090;13.134;13.194;...
            13.234;13.283;13.330;13.375;13.421;13.465;13.520;13.565;...
            13.615;13.660;13.710;13.758;13.814;13.858;13.905;13.950;...
            13.996;14.046;14.089;14.133;14.186;14.230;14.272;14.332;...
            14.373;14.419;14.471;14.520;14.567;14.609;14.653;14.706;...
            14.756;14.821;14.849;14.896;14.947;14.992;15.024;15.082;...
            15.122;15.176;15.232;15.277;15.347;15.390;15.472;15.496;...
            15.534;15.556];
        nindex=spline(wavelength,nalu_data,lambda);
        kindex=spline(wavelength,kalu_data,lambda);
        nindex=nindex+1i*kindex;
    elseif indice==5    % SiO2 (Dispersion formula by Malitson 1965)
        nindex=sqrt(1+0.6961663*lambda.^2./(lambda.^2-68.4043^2)+...
            0.4079426*lambda.^2./(lambda.^2-116.2414^2)+...
            0.8974794*lambda.^2./(lambda.^2-9896.161^2));
    elseif indice==6    % TiO2 (ordnary ray) (Dispersion formula by Devore 1951)
        nindex=sqrt(5.913 + 244100./(lambda.^2-80300));
    elseif indice==7    % TiO2 (extraordnary ray) (Dispersion formula by Devore 1951)
        nindex=sqrt(7.197 + 332200./(lambda.^2-84300));
    elseif indice==8    % TiO2 (average) (Dispersion formula by Devore 1951)
        nindex=0.5*(sqrt(5.913 + 244100./(lambda.^2-80300)) + sqrt(7.197 + 332200./(lambda.^2-84300)));
    elseif indice==9    % External file
        if volume==1
            Data=load(['IndexFiles/' core_filename]);
        elseif volume==2
            Data=load(['IndexFiles/' shell_filename]);
        end
        wavelength=Data(:,1);
        nfile_data=Data(:,2);
        kfile_data=Data(:,3);
        nindex=spline(wavelength,nfile_data,lambda);
        kindex=spline(wavelength,kfile_data,lambda);
        nindex=nindex+1i*kindex;
    elseif indice==10    % Custom
        if volume==1
            nindex=str2double(get(h.NPCore_n_edit,'String'))*ones(size(lambda));
            kindex=str2double(get(h.NPCore_k_edit,'String'))*ones(size(lambda));
            nindex=nindex+1i*kindex;
        elseif volume==2
            nindex=str2double(get(h.NPShell_n_edit,'String'))...
                *ones(size(lambda));
            kindex=str2double(get(h.NPShell_k_edit,'String'))...
                *ones(size(lambda));
            nindex=nindex+1i*kindex;
        end
    end
end



function [nindex] = get_medium_nindex(indice,lambda)
    if indice==1    % Water (Dispersion formula by Daimon et al. 20degC 2007)
        nindex=sqrt(1+0.5684027565*lambda.^2./(lambda.^2-71.42709368^2)+...
            0.1726177391*lambda.^2./(lambda.^2-134.9501366^2)+...
            0.02086189578*lambda.^2./(lambda.^2-161.8864507^2)+...
            0.1130748688*lambda.^2./(lambda.^2-3270.768596^2));
    elseif indice==2    % Air
        nindex=ones(size(lambda));
    elseif indice==3    % Custom
        nindex=str2double(get(h.Medium_n_edit,'String'))*ones(size(lambda));
    end
end



function [k, c, rho] = get_NPData(material)
    if strcmp(material,'Au')
        k = 3.15e-7;        % In W/(nm*K)
        c = 0.129;          % In J/(g*K)
        rho = 1.93e-20;     % In g/nm
    elseif strcmp(material,'Ag')
        k = 4.30e-7;        % In W/(nm*K)
        c = 0.235;          % In J/(g*K)
        rho = 1.049e-20;    % In g/nm
    elseif strcmp(material,'SiO2')
        k = 1.40e-9;        % In W/(nm*K)
        c = 0.703;          % In J/(g*K)
        rho = 2.20e-21;     % In g/nm
    elseif strcmp(material,'Water')
        k = 6.084e-10;      % In W/(nm*K)
        c = 4.1813;         % In J/(g*K)
        rho = 1e-21;        % In g/nm
    end
end



function [G] = get_NPDataG(material1, material2)
    MatList={'Au','AuAg Alloy','Ag','Cu','Al','SiO2', 'TiO2(o)', 'TiO2(e)', 'TiO2(avg)', 'Water', 'Air'};
    Indx1=find(strcmp(material1,MatList));
    Indx2=find(strcmp(material2,MatList));
    
    GMatrix=zeros(length(MatList),length(MatList));
    G=GMatrix(Indx1,Indx2);
end



function [TransImg, ScatImg] = ScatExtSpectra2sRGB(lambda, ExtSpectrum, sca)
% Computes the equivalent sRGB values for the transmitted and scattered
% colors of NPs illuminated by a standard D65 illuminant white source.
%
% Calculation is based on the determination of the transmitted spectrum and
% the scattered spectrum using D65 illuminant and the extinction and
% scattering spectra of the NPs. Conversion to CIEXYZ colorspace is
% computed using CIE1931 standard observer values. A standard sRGB
% conversion is used to find the sRGB values.

lambdaref=380:1:780;

CIEdata=[380 0.001368 0.000039 0.006450; 385 0.002236 0.000064 0.010550;...
    390 0.004243 0.000120 0.020050; 395 0.007650 0.000217 0.036210;...
    400 0.014310 0.000396 0.067850; 405 0.023190 0.000640 0.110200;...
    410 0.043510 0.001210 0.207400; 415 0.077630 0.002180 0.371300;...
    420 0.134380 0.004000 0.645600; 425 0.214770 0.007300 1.039050;...
    430 0.283900 0.011600 1.385600; 435 0.328500 0.016840 1.622960;...
    440 0.348280 0.023000 1.747060; 445 0.348060 0.029800 1.782600;...
    450 0.336200 0.038000 1.772110; 455 0.318700 0.048000 1.744100;...
    460 0.290800 0.060000 1.669200; 465 0.251100 0.073900 1.528100;...
    470 0.195360 0.090980 1.287640; 475 0.142100 0.112600 1.041900;...
    480 0.095640 0.139020 0.812950; 485 0.057950 0.169300 0.616200;...
    490 0.032010 0.208020 0.465180; 495 0.014700 0.258600 0.353300;...
    500 0.004900 0.323000 0.272000; 505 0.002400 0.407300 0.212300;...
    510 0.009300 0.503000 0.158200; 515 0.029100 0.608200 0.111700;...
    520 0.063270 0.710000 0.078250; 525 0.109600 0.793200 0.057250;...
    530 0.165500 0.862000 0.042160; 535 0.225750 0.914850 0.029840;...
    540 0.290400 0.954000 0.020300; 545 0.359700 0.980300 0.013400;...
    550 0.433450 0.994950 0.008750; 555 0.512050 1.000000 0.005750;...
    560 0.594500 0.995000 0.003900; 565 0.678400 0.978600 0.002750;...
    570 0.762100 0.952000 0.002100; 575 0.842500 0.915400 0.001800;...
    580 0.916300 0.870000 0.001650; 585 0.978600 0.816300 0.001400;...
    590 1.026300 0.757000 0.001100; 595 1.056700 0.694900 0.001000;...
    600 1.062200 0.631000 0.000800; 605 1.045600 0.566800 0.000600;...
    610 1.002600 0.503000 0.000340; 615 0.938400 0.441200 0.000240;...
    620 0.854450 0.381000 0.000190; 625 0.751400 0.321000 0.000100;...
    630 0.642400 0.265000 0.000050; 635 0.541900 0.217000 0.000030;...
    640 0.447900 0.175000 0.000020; 645 0.360800 0.138200 0.000010;...
    650 0.283500 0.107000 0.000000; 655 0.218700 0.081600 0.000000;...
    660 0.164900 0.061000 0.000000; 665 0.121200 0.044580 0.000000;...
    670 0.087400 0.032000 0.000000; 675 0.063600 0.023200 0.000000;...
    680 0.046770 0.017000 0.000000; 685 0.032900 0.011920 0.000000;...
    690 0.022700 0.008210 0.000000; 695 0.015840 0.005723 0.000000;...
    700 0.011359 0.004102 0.000000; 705 0.008111 0.002929 0.000000;...
    710 0.005790 0.002091 0.000000; 715 0.004109 0.001484 0.000000;...
    720 0.002899 0.001047 0.000000; 725 0.002049 0.000740 0.000000;...
    730 0.001440 0.000520 0.000000; 735 0.001000 0.000361 0.000000;...
    740 0.000690 0.000249 0.000000; 745 0.000476 0.000172 0.000000;...
    750 0.000332 0.000120 0.000000; 755 0.000235 0.000085 0.000000;...
    760 0.000166 0.000060 0.000000; 765 0.000117 0.000042 0.000000;...
    770 0.000083 0.000030 0.000000; 775 0.000059 0.000021 0.000000;...
    780 0.000042 0.000015 0.000000];
ConvMatrix=[3.2406 -1.5372 -0.4986; -0.9689 1.8758 0.0415; ...
    0.0557 -0.2040 1.0570];
D65=[300   0.034100; 305   1.664300; 310   3.294500; 315  11.765200;...
    320  20.236000; 325  28.644700; 330  37.053500; 335  38.501100;...
    340  39.948800; 345  42.430200; 350  44.911700; 355  45.775000;...
    360  46.638300; 365  49.363700; 370  52.089100; 375  51.032300;...
    380  49.975500; 385  52.311800; 390  54.648200; 395  68.701500;...
    400  82.754900; 405  87.120400; 410  91.486000; 415  92.458900;...
    420  93.431800; 425  90.057000; 430  86.682300; 435  95.773600;...
    440 104.865000; 445 110.936000; 450 117.008000; 455 117.410000;...
    460 117.812000; 465 116.336000; 470 114.861000; 475 115.392000;...
    480 115.923000; 485 112.367000; 490 108.811000; 495 109.082000;...
    500 109.354000; 505 108.578000; 510 107.802000; 515 106.296000;...
    520 104.790000; 525 106.239000; 530 107.689000; 535 106.047000;...
    540 104.405000; 545 104.225000; 550 104.046000; 555 102.023000;...
    560 100.000000; 565  98.167100; 570  96.334200; 575  96.061100;...
    580  95.788000; 585  92.236800; 590  88.685600; 595  89.345900;...
    600  90.006200; 605  89.802600; 610  89.599100; 615  88.648900;...
    620  87.698700; 625  85.493600; 630  83.288600; 635  83.493900;...
    640  83.699200; 645  81.863000; 650  80.026800; 655  80.120700;...
    660  80.214600; 665  81.246200; 670  82.277800; 675  80.281000;...
    680  78.284200; 685  74.002700; 690  69.721300; 695  70.665200;...
    700  71.609100; 705  72.979000; 710  74.349000; 715  67.976500;...
    720  61.604000; 725  65.744800; 730  69.885600; 735  72.486300;...
    740  75.087000; 745  69.339800; 750  63.592700; 755  55.005400;...
    760  46.418200; 765  56.611800; 770  66.805400; 775  65.094100;...
    780  63.382800; 785  63.843400; 790  64.304000; 795  61.877900;...
    800  59.451900; 805  55.705400; 810  51.959000; 815  54.699800;...
    820  57.440600; 825  58.876500; 830  60.312500];

Xref=spline(CIEdata(:,1), CIEdata(:,2), lambdaref);
Yref=spline(CIEdata(:,1), CIEdata(:,3), lambdaref);
Zref=spline(CIEdata(:,1), CIEdata(:,4), lambdaref);
D65ref=spline(D65(:,1), D65(:,2), lambdaref);
ConvRatio=sum(D65ref.*Yref);

ExtSpectrumSpline=spline(lambda, ExtSpectrum, lambdaref);
XtSpectrum=D65ref.*exp(-ExtSpectrumSpline);

ScatSpectrumSpline=spline(lambda, sca/max(sca), lambdaref);
ScSpectrum=D65ref.*ScatSpectrumSpline;

XtXYZ(1,1)=sum(XtSpectrum.*Xref)/ConvRatio;
XtXYZ(2,1)=sum(XtSpectrum.*Yref)/ConvRatio;
XtXYZ(3,1)=sum(XtSpectrum.*Zref)/ConvRatio;

ScXYZ(1,1)=sum(ScSpectrum.*Xref)/ConvRatio;
ScXYZ(2,1)=sum(ScSpectrum.*Yref)/ConvRatio;
ScXYZ(3,1)=sum(ScSpectrum.*Zref)/ConvRatio;

XtRGB=ConvMatrix*XtXYZ;
ScRGB=ConvMatrix*ScXYZ;

% Clipping values within [0 1]
XtRGBlin=max(min(XtRGB,1),0);
ScRGBlin=max(min(ScRGB,1),0);

XtsRGB=12.92*XtRGBlin.*(1-sign(XtRGBlin-0.0031308))/2 ...
    + 1.055*(XtRGBlin.*(1+sign(XtRGBlin-0.0031308))/2).^(1/2.4) ...
    - 0.055*(1+sign(XtRGBlin-0.0031308))/2;
% XtsRGB=XtsRGB;
Xthsv=rgb2hsv(XtsRGB');

ScsRGB=12.92*ScRGBlin.*(1-sign(ScRGBlin-0.0031308))/2 ...
    + 1.055*(ScRGBlin.*(1+sign(ScRGBlin-0.0031308))/2).^(1/2.4) ...
    - 0.055*(1+sign(ScRGBlin-0.0031308))/2;
ScsRGB=255*ScsRGB;


TransImghsv=Xthsv(1)*ones(size(S));
TransImghsv(:,:,2)=Xthsv(2)*double(S)/255;
TransImghsv(:,:,3)=double(V)/255;
TransImg=hsv2rgb(TransImghsv);

[x,y]=meshgrid(-50:1:50);
r=sqrt(x.^2+y.^2)/8;
PSF=(2*besselj(1,r)./r).^2;
PSF((length(x)+1)/2,(length(y)+1)/2)=1;
ScatImg=uint8(PSF*ScsRGB(1));
ScatImg(:,:,2)=uint8(PSF*ScsRGB(2));
ScatImg(:,:,3)=uint8(PSF*ScsRGB(3));
end



function AbsorptionCalculation
    NP_Type=get(h.NPType_popup,'Value');
    if NP_Type==1
        diam=str2double(get(h.NPCoreDiam_edit,'String'));
        lambda=(300:1:1000)';
        core_index_selected=get(h.NPCoreMaterial_popup,'Value');
        [core_nindex]=get_nindex(1,core_index_selected,lambda);
        medium_index_selected=get(h.Medium_popup,'Value');
        [medium_nindex]=get_medium_nindex(medium_index_selected,lambda);
        [abs,~,~]=NFMieCoated(diam,[core_nindex,medium_nindex],lambda,1);
        
        LambdaLaser = str2double(get(h.LambdaLaser_edit,'String'));
        [core_nindex]=get_nindex(1,core_index_selected,LambdaLaser);
        [medium_nindex]=get_medium_nindex(medium_index_selected,LambdaLaser);
        [NPabs,~,~]=NFMieCoated(diam,[core_nindex,medium_nindex],LambdaLaser,1);
    else
        CoreDiam=str2double(get(h.NPCoreDiam_edit,'String'));
        ShellThick=str2double(get(h.NPShellThick_edit,'String'));
        diam=[CoreDiam CoreDiam+2*ShellThick];
        lambda=(300:1:1000)';
        core_index_selected=get(h.NPCoreMaterial_popup,'Value');
        [core_nindex]=get_nindex(1,core_index_selected,lambda);
        shell_index_selected=get(h.NPShellMaterial_popup,'Value');
        [shell_nindex]=get_nindex(2,shell_index_selected,lambda);
        medium_index_selected=get(h.Medium_popup,'Value');
        [medium_nindex]=get_medium_nindex(medium_index_selected,lambda);
        [abs,~,~]=NFMieCoated(diam,...
            [core_nindex,shell_nindex,medium_nindex],lambda,1);
        
        LambdaLaser = str2double(get(h.LambdaLaser_edit,'String'));
        [core_nindex]=get_nindex(1,core_index_selected,LambdaLaser);
        [shell_nindex]=get_nindex(2,shell_index_selected,LambdaLaser);
        [medium_nindex]=get_medium_nindex(medium_index_selected,LambdaLaser);
        [NPabs,~,~]=NFMieCoated(diam,...
            [core_nindex,shell_nindex,medium_nindex],LambdaLaser,1);
    end
    
    axes(h.NPAbs_axes);
    hAbs_plot=plot(lambda,abs,'linewidth',1.5,'color',[0 0 1]);
    hold on;
    LambdaVbar = str2double(get(h.LambdaLaser_edit,'String'));
    hVbar_plot=plot([LambdaVbar LambdaVbar],[0 1.1*max(abs)],'linewidth',1,'linestyle',':','color',[0 0 0]);
    hold off;
    axis([300, 1000, 0 1.1*max(abs)]);
    set(gca,'FontSize',10);
    grid on;
    xlabel('Wavelength (nm)','BackgroundColor',[0.9 0.9 0.9]);
    ylabel('\sigma_a_b_s (nm)','BackgroundColor',[0.9 0.9 0.9]);
    
    set(h.AbsMax_text,'String',['Absorption cross-section @'...
        num2str(LambdaLaser) 'nm = ' num2str(NPabs,'%.3g') 'nm']);
    
end



function TraceTempResults(SpatialIndex, TimeIndex)
    NPRadius = 0.5*NPData.diameters(end);
    maxT=max(NPData.T(:));
    minT=min(NPData.T(:)); 
    
    axes(h.SpectraAndTemporal_axes);
    hTempProfile=plot(NPData.time,NPData.T(SpatialIndex,:),'linewidth',1.5,'color',[0 0 1]);
    hold on;
    plot([NPData.time(TimeIndex) NPData.time(TimeIndex)], [0 1.1*maxT],'linewidth',1,'linestyle',':','color',[1 0 0]);
    hold off;
    axis([0, NPData.time(end), 0 1.1*maxT]);
    set(gca,'FontSize',10);
    grid on;
    xlabel('Time (s)');
    ylabel('\Delta T (K)','BackgroundColor',[0.9 0.9 0.9]);
    
    axes(h.eFieldAndSpatial_axes);
    hSpatProfile=plot([NPRadius NPRadius], [0 1.1*maxT],'linewidth',1,'linestyle','--','color',[0 0.6 0]);
    hold on;
    plot(NPData.r,NPData.T(:,TimeIndex),'linewidth',1.5,'color',[0 0 1]);
    plot([NPData.r(SpatialIndex) NPData.r(SpatialIndex)], [0 1.1*maxT],'linewidth',1,'linestyle',':','color',[1 0 0]);
    hold off;
    axis([0, 10*NPRadius, 0 1.1*maxT]);
    set(gca,'FontSize',10);
    grid on;
    xlabel('Radial Position (nm)');
    ylabel('\Delta T (K)','BackgroundColor',[0.9 0.9 0.9]);
    
    [X,Y]=meshgrid(linspace(-2*NPRadius,2*NPRadius,500));
    R=sqrt(X.^2+Y.^2);
    colormap(parula(1024));
%     TNP=spline(r,T(:,TimeIndex),R);         % There is a bug when using spline because values of r are too close at the interface
    TNP = interp1(NPData.r,NPData.T(:,TimeIndex),R);      % Simple interpolation works better
    axes(h.ImageTemp_axes);
    pcolor(X,Y,TNP); shading flat; axis equal;
    hold on
    plot(NPRadius*cos(0:pi/60:2*pi),NPRadius*sin(0:pi/60:2*pi),'linestyle',':','color',[0 0 0]);
    plot(NPData.r(SpatialIndex),0,'.','color',[1 0 0]);
    hold off
    caxis([minT, maxT]);
    set(h.ImageTemp_axes,'Visible','off');
    colorbar;
end














function NPPhotothermiaInterface


function Save_button_Callback(hObject, eventdata)
    SpatialIndex = round(get(h.PositionVisualization_slider,'value')+1);
    TimeIndex=round(get(h.TimeVisualization_slider,'value')+1);
    
    filename=get(Save_edit,'String');
    
    fSpat=fopen([filename '_spatial_profile.txt'],'w');
    fprintf(fSpat,'Spatial profile\r\n');
    fprintf(fSpat,'=======================================================\r\n');
    fprintf(fSpat,['NP diameter: ' NPDiam_edit.String ' nm\r\n']);
    fprintf(fSpat,['Irradiation wavelength: ' Lambda_edit.String ' nm\r\n']);
    fprintf(fSpat,['Absorption cross section: ' num2str(NPabs,'%.3g') ' nm\r\n']);
    fprintf(fSpat,['Laser intensity: ' h.LaserIntensity_edit.String ' W/cm\r\n']);
    fprintf(fSpat,['Pulse duration: ' h.PulseTime_edit.String ' s\r\n']);
    fprintf(fSpat,['Time of analysis: ' num2str(time(TimeIndex),'%.3g') ' s\r\n']);
    fprintf(fSpat,'=======================================================\r\n');
    fprintf(fSpat,'%s\t %s\t \r\n','Position (nm)',' Temperature increase (K)');
    for k=1:length(r)
        fprintf(fSpat,'%#13.3f\t %#24.5g \r\n',[r(k) T(k,TimeIndex)]);
    end
    fclose(fSpat);
    
    fTime=fopen([filename '_temporal_profile.txt'],'w');
    fprintf(fTime,'Temporal profile\r\n');
    fprintf(fTime,'=======================================================\r\n');
    fprintf(fTime,['NP diameter: ' NPDiam_edit.String ' nm\r\n']);
    fprintf(fTime,['Irradiation wavelength: ' Lambda_edit.String ' nm\r\n']);
    fprintf(fTime,['Absorption cross section: ' num2str(NPabs,'%.3g') ' nm\r\n']);
    fprintf(fTime,['Laser intensity: ' h.LaserIntensity_edit.String ' W/cm\r\n']);
    fprintf(fTime,['Pulse duration: ' h.PulseTime_edit.String ' s\r\n']);
    fprintf(fTime,['Position of analysis: ' num2str(r(SpatialIndex),'%.5g') ' nm\r\n']);
    fprintf(fTime,'=======================================================\r\n');
    fprintf(fTime,'%8s\t %24s\t \r\n','Time (s)',' Temperature increase (K)');
    for k=1:length(time)
        fprintf(fTime,'%#8.5g\t %#24.5g \r\n',[time(k) T(SpatialIndex,k)]);
    end
    fclose(fTime);
end

end



end